Attribute VB_Name = "mjStrip"
Option Explicit

'********************************
' This module provides the core routines
' for jStrip.
'
' The FileTime routines & related declares,
' types, and constants are from VBNet.
' VBNet is a great site.  Visit them at
' http://www.mvps.org/vbnet
'
' The rest of the code is (C)1998 David Crowell
'
' Feel free to use any of these routines in
' your own apps.
'********************************

Public bCancelFlag As Byte
Public Options As String
Public ErrorString As String
Public OriginalSize As Long
Public ProcessedSize As Long

Public Const OKAY As Long = 0
Public Const ERROR As Long = -1
Public Const DONE As Long = -2
Public Const PROBLEM As Long = -3
Public Const ISREADONLY As Long = -4
Public Const NOCHANGE As Long = -5
Public Const OFS_MAXPATHNAME = 260
Public Const OF_READWRITE = &H2

Public Type OFSTRUCT
   cBytes      As Byte
   fFixedDisk  As Byte
   nErrCode    As Integer
   Reserved1   As Integer
   Reserved2   As Integer
   szPathName(OFS_MAXPATHNAME) As Byte
End Type

Public Type FILETIME
  dwLowDateTime     As Long
  dwHighDateTime    As Long
End Type

Public Declare Function GetFileTime Lib "kernel32" _
   (ByVal hFile As Long, lpCreationTime As FILETIME, _
    lpLastAccessTime As FILETIME, _
    lpLastWriteTime As FILETIME) As Long
    
Public Declare Function SetFileTime Lib "kernel32" _
   (ByVal hFile As Long, _
    lpCreationTime As FILETIME, _
    lpLastAccessTime As FILETIME, _
    lpLastWriteTime As FILETIME) As Long

Public Declare Function OpenFile Lib "kernel32" _
   (ByVal lpFileName As String, _
    lpReOpenBuff As OFSTRUCT, _
    ByVal wStyle As Long) As Long
    
Public Declare Function CloseHandle Lib "kernel32" _
   (ByVal hFile As Long) As Long
   
Public Declare Function ShellExecute Lib "shell32.dll" _
    Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal _
    lpOperation As String, ByVal lpFile As String, ByVal _
    lpParameters As String, ByVal lpDirectory As String, _
    ByVal nShowCmd As Long) As Long



Private iFN As Integer          ' open file number
Private inBuf() As Byte         ' byte array for unprocessed file
Private outBuf() As Byte       ' byte array for processed file
Private CurPos As Long        ' current position in input buffer
Private PathToFile As String

'***********************************
' Read the input file, and return the size (or error code)
'***********************************

Private Function ReadFile() As Long

    On Error GoTo handle:

    iFN = FreeFile
    Open PathToFile For Binary As iFN
    ReadFile = LOF(iFN)                     ' read file size
    ReDim inBuf(ReadFile + 10)
    ReDim outBuf(ReadFile + 10)
    Get #iFN, 1, inBuf()                    ' read the file into a byte array
    Close iFN
    
Exit Function

handle:
    ReadFile = ERROR
    Close iFN
End Function

'***********************************
' Write the output file, return result code.
'***********************************

Private Function WriteFile()

    On Error GoTo handle:
    
    ReDim Preserve outBuf(ProcessedSize - 1)
    
    If CheckReadOnly(PathToFile) Then       ' Check if File is ReadOnly
        WriteFile = ISREADONLY
        Exit Function
    End If
    
    Kill PathToFile                            ' Delete the original file
    
    iFN = FreeFile                            ' open new file and write
    Open PathToFile For Binary As iFN
    Put #iFN, , outBuf()
    Close iFN
    
    WriteFile = OKAY                          ' Okey dokey
    
Exit Function

handle:
    WriteFile = ERROR
    Close iFN
End Function

'***********************************
' Scan the array for the beginning of the JPEG
' data.  Return the position.
'***********************************

Private Function FindJpgHeader() As Long

    Dim Pos As Long
    Do
    
        If inBuf(Pos) = &HFF And inBuf(Pos + 1) = &HD8 And inBuf(Pos + 2) = &HFF Then
            FindJpgHeader = Pos              ' return position
            Exit Do
        End If
        
        If Pos >= OriginalSize Then
            FindJpgHeader = ERROR               ' can't find header
            Exit Do
        End If
        
        Pos = Pos + 1
        
    Loop
End Function

'***********************************
' Copy the needed portions, and skip the rest.
' Return status.
' If you really need to understand how this
' works, you should download some information
' on the JPEG file format.  I believe that
' http://www.wotsit.org has what you need.
'***********************************

Private Function GetMarkers() As Long

    Dim lSkip As Long
    Dim lTemp As Long
    Dim bFlag As Byte
    
    Select Case inBuf(CurPos)
    
        Case &HD8
            WriteArray &HFF
            WriteArray &HD8
            WriteArray &HFF
            lSkip = 2
            
        Case &HE0, &HDB, &HC0 To &HCB, &HDD
            lSkip = Mult(inBuf(CurPos + 2), inBuf(CurPos + 1)) + 1
            If lSkip + CurPos >= OriginalSize Then GoTo Oops
            For lTemp = CurPos To CurPos + lSkip
                WriteArray inBuf(lTemp)
            Next lTemp
            
        Case &HDA
            bFlag = 1
            Do
                WriteArray inBuf(CurPos)
                If inBuf(CurPos + 1) = &HFF And inBuf(CurPos + 2) = &HD9 Then Exit Do
                CurPos = CurPos + 1
                If CurPos > OriginalSize Then
                    bFlag = 2
                    Exit Do
                End If
            Loop
            WriteArray &HFF
            WriteArray &HD9
            
        Case Else
            lSkip = Mult(inBuf(CurPos + 2), inBuf(CurPos + 1)) + 1
            If lSkip + CurPos > OriginalSize Then GoTo Oops
            
    End Select
    CurPos = CurPos + lSkip
    Do
        If inBuf(CurPos) <> &HFF Then Exit Do
        CurPos = CurPos + 1
        If CurPos > OriginalSize Then GoTo Oops
    Loop
    If bFlag = 0 Then GetMarkers = OKAY
    If bFlag = 1 Then GetMarkers = DONE
    If bFlag = 2 Then GetMarkers = PROBLEM
Exit Function
Oops:
GetMarkers = ERROR
End Function

'***********************************
' Add another byte to the output array, and
' increment the count.
'***********************************

Private Sub WriteArray(bData As Byte)
    outBuf(ProcessedSize) = bData
    ProcessedSize = ProcessedSize + 1
End Sub

'***********************************
' Ties together the other functions to process
' one file.
'***********************************

Public Function Strip(sFileIn As String) As Long

    Dim lret As Long
    PathToFile = sFileIn
    
    OriginalSize = 0
    ProcessedSize = 0
    CurPos = 0
    
    
    OriginalSize = ReadFile()                        ' returns filesize or ERROR
    If OriginalSize = ERROR Then
        ErrorString = "Couldn't open input file."
        GoTo UhOh
    End If
    
    CurPos = FindJpgHeader()                                ' find the jpg header
    If CurPos = ERROR Then
        ErrorString = "Not a valid JPEG file.  File unchanged."
        GoTo UhOh
    End If
    CurPos = CurPos + 1
    
    lret = 0
    Do Until lret = DONE Or lret = ERROR Or lret = PROBLEM
        lret = GetMarkers()                         ' copy needed data
    Loop
    If lret = ERROR Then
        ErrorString = "Problem parsing file.  File unchanged."
        GoTo UhOh
    End If
    
    If ProcessedSize <> OriginalSize Then
        lret = WriteFile()                               ' write output file
        If lret = ERROR Then
            ErrorString = "Could not write output file."
            GoTo UhOh
        End If
        If lret = ISREADONLY Then
            ErrorString = "Cannot overwrite read-only file."
            GoTo UhOh
        End If
        Strip = OKAY
    Else
        Strip = NOCHANGE
    End If
    
Exit Function
UhOh:                                                   ' Houston, we have a problem
    Strip = ERROR
End Function

'***********************************
' Clear the Read-Only bit if the FixReadOnly
' property is true.  This is called from WriteFile
'***********************************

Public Function CheckReadOnly(path As String) As Boolean

    On Error GoTo handle
    
    Dim x As VbFileAttribute
    
    x = GetAttr(path)
    If x And vbReadOnly Then                ' is file read-only?
    
        If Mid(Options, 1, 1) = 1 Then         ' check if user wants to change
            SetAttr path, vbNormal
        Else
            CheckReadOnly = True
        End If
        
    End If
    
Exit Function
handle:
    CheckReadOnly = True
End Function

'***********************************
' Set the last modified time & date.
'***********************************

Public Sub WriteFileTime(FName As String, ft As FILETIME)

   Dim r As Long
   Dim hFile As Long
   
   Dim OFS As OFSTRUCT
   Dim Created As FILETIME
   Dim Accessed As FILETIME
   Dim Modified As FILETIME
    
  'open the file
   hFile = OpenFile(FName, OFS, OF_READWRITE)
   
  'set the created, modified, and accessed dates
   r = SetFileTime(hFile, ft, ft, ft)
   
  'clean up by closing the file
   r = CloseHandle(hFile)

End Sub

'***********************************
' Read the last modified time & date.
'***********************************

Public Function ReadFileTime(FName As String) As FILETIME

   Dim r As Long
   Dim hFile As Long
   
   Dim OFS As OFSTRUCT
   Dim Created As FILETIME
   Dim Accessed As FILETIME
   Dim Modified As FILETIME
    
  'open the file
   hFile = OpenFile(FName, OFS, OF_READWRITE)
   
  'get the FILETIME info for the created,
  'accessed and last write info
   r = GetFileTime(hFile, Created, Accessed, Modified)
   
  'clean up by closing the file
   r = CloseHandle(hFile)
   
   'return the needed value
   ReadFileTime = Modified
   
End Function

'***********************************
' This short little routine is to take two byte
' values, and return a long.
'***********************************

Private Function Mult(lsb As Byte, msb As Byte) As Long
    Mult = CLng(lsb) + (CLng(msb) * 256&)
End Function

